@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_cache.c,v 3.140 2007/04/02 22:57:10 rasmus Exp $ */
+/* $Id: apc_cache.c,v 3.145 2007/10/05 23:06:56 gopalv Exp $ */
 
 #include "apc_cache.h"
 #include "apc_lock.h"
@@ -97,6 +97,7 @@ typedef struct local_slot_t local_slot_t;
 struct local_slot_t {
     slot_t *original;           /* the original slot in shm */
     int num_hits;               /* number of hits */
+    time_t creation_time;       /* creation time */
     apc_cache_entry_t *value;   /* shallow copy of slot->value */
     local_slot_t *next;         /* only for dead list */
 };
@@ -306,6 +307,9 @@ apc_cache_t* apc_cache_create(int size_hint, int gc_ttl, int ttl)
     cache_size = sizeof(header_t) + num_slots*sizeof(slot_t*);
 
     cache->shmaddr = apc_sma_malloc(cache_size);
+    if(!cache->shmaddr) {
+        apc_eprint("Unable to allocate shared memory for cache structures.  (Perhaps your shared memory size isn't large enough?). ");
+    }
     memset(cache->shmaddr, 0, cache_size);
 
     cache->header = (header_t*) cache->shmaddr;
@@ -712,11 +716,10 @@ int apc_cache_make_file_key(apc_cache_key_t* key,
                        time_t t
 					   TSRMLS_DC)
 {
-    static char canon_path[MAXPATHLEN];
     struct stat *tmp_buf=NULL;
     struct apc_fileinfo_t fileinfo = { {0}, };
     int len;
-	
+
     assert(key != NULL);
 
     if (!filename || !SG(request_info).path_translated) {
@@ -734,12 +737,18 @@ int apc_cache_make_file_key(apc_cache_key_t* key,
             key->mtime = t;
             key->type = APC_CACHE_KEY_FPFILE;
         } else {
-            if(!realpath(filename, canon_path)) {
-                fprintf(stderr, "realpath failed to canonicalize %s - bailing\n", filename);
+            if (apc_search_paths(filename, include_path, &fileinfo) != 0) {
+                apc_wprint("apc failed to locate %s - bailing", filename);
                 return 0;
             }
-            key->data.fpfile.fullpath = canon_path;
-            key->data.fpfile.fullpath_len = strlen(canon_path);
+
+            if(!realpath(fileinfo.fullpath, APCG(canon_path))) {
+                apc_wprint("realpath failed to canonicalize %s - bailing", filename);
+                return 0;
+            }
+
+            key->data.fpfile.fullpath = APCG(canon_path);
+            key->data.fpfile.fullpath_len = strlen(APCG(canon_path));
             key->mtime = t;
             key->type = APC_CACHE_KEY_FPFILE;
         }
@@ -853,7 +862,6 @@ apc_cache_entry_t* apc_cache_make_file_entry(const char* filename,
     entry->type = APC_CACHE_ENTRY_FILE;
     entry->ref_count = 0;
     entry->mem_size = 0;
-    entry->autofiltered = 0;
     entry->local = 0;
     return entry;
 }
@@ -995,7 +1003,6 @@ apc_cache_entry_t* apc_cache_make_user_entry(const char* info, int info_len, con
     entry->type = APC_CACHE_ENTRY_USER;
     entry->ref_count = 0;
     entry->mem_size = 0;
-    entry->autofiltered = 0;
     entry->local = 0;
     return entry;
 }
@@ -1171,7 +1178,7 @@ void apc_cache_write_unlock(apc_cache_t* cache)
 #endif
 
 /* {{{ make_local_slot */
-static local_slot_t* make_local_slot(apc_local_cache_t* cache, local_slot_t* lslot, slot_t* slot) 
+static local_slot_t* make_local_slot(apc_local_cache_t* cache, local_slot_t* lslot, slot_t* slot, time_t t) 
 {
     apc_cache_entry_t* value;
 
@@ -1181,7 +1188,8 @@ static local_slot_t* make_local_slot(apc_local_cache_t* cache, local_slot_t* lsl
 
     lslot->original = slot;
     lslot->value = value;
-    lslot->num_hits++;
+    lslot->num_hits = 0;
+    lslot->creation_time = t;
 
     return lslot; /* for what joy ? ... consistency */
 }
@@ -1233,10 +1241,11 @@ void apc_local_cache_cleanup(apc_local_cache_t* cache) {
     
     int i;
     for(i = 0; i < cache->num_slots; i++) {
-        slot_t * slot = cache->slots[i].original;
-        if((slot && slot->access_time < (t - cache->ttl)) ||
+        lslot = &cache->slots[i];
+        /* every slot lives for exactly TTL seconds */
+        if((lslot->original && lslot->creation_time < (t - cache->ttl)) ||
                 cache->generation != cache->shmcache->header->expunges) {
-            free_local_slot(cache, &cache->slots[i]);
+            free_local_slot(cache, lslot);
         }
     }
 
@@ -1292,9 +1301,15 @@ apc_cache_entry_t* apc_local_cache_find(apc_local_cache_t* cache, apc_cache_key_
                 free_local_slot(cache, lslot);
                 goto not_found;
             }
+            cache->num_hits++;
+            lslot->num_hits++;
+            lslot->original->access_time = t; /* unlocked write, but last write wins */
             return lslot->value;
         } else if(key.type == APC_CACHE_KEY_FPFILE) {
             if(!memcmp(slot->key.data.fpfile.fullpath, key.data.fpfile.fullpath, key.data.fpfile.fullpath_len+1)) {
+                cache->num_hits++;
+                lslot->num_hits++;
+                lslot->original->access_time = t; /* unlocked write, but last write wins */
                 return lslot->value;
             }
         }
@@ -1311,7 +1326,7 @@ not_found:
     /* i.e maintain a sort of top list */
     if(lslot->original == NULL || (lslot->original->num_hits + lslot->num_hits)  < slot->num_hits) {
         free_local_slot(cache, lslot);
-        make_local_slot(cache, lslot, slot); 
+        make_local_slot(cache, lslot, slot, t); 
         return lslot->value;
     }
     return slot->value;
