@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_compile.c,v 3.85 2007/03/28 07:35:55 gopalv Exp $ */
+/* $Id: apc_compile.c,v 3.87 2007/08/25 13:09:13 gopalv Exp $ */
 
 #include "apc_compile.h"
 #include "apc_globals.h"
@@ -1266,11 +1266,10 @@ zend_op_array* apc_copy_op_array(zend_op_array* dst, zend_op_array* src, apc_mal
             if((zo->opcode == ZEND_INCLUDE_OR_EVAL) && 
                 (zo->op1.op_type == IS_CONST && zo->op1.u.constant.type == IS_STRING)) {
                 /* constant includes */
-                if(!IS_ABSOLUTE_PATH(Z_STRVAL_P(&zo->op1.u.constant),len)) { 
+                if(!IS_ABSOLUTE_PATH(Z_STRVAL_P(&zo->op1.u.constant),Z_STRLEN_P(&zo->op1.u.constant))) { 
                     if (apc_search_paths(Z_STRVAL_P(&zo->op1.u.constant), PG(include_path), &fileinfo) == 0) {
-                        if((IS_ABSOLUTE_PATH(fileinfo.fullpath, strlen(fileinfo.fullpath)) && (fullpath = fileinfo.fullpath))
-                                || (fullpath = realpath(fileinfo.fullpath, canon_path))) {
-                            /* is either an absolute path or it goes through a realpath() */
+                        if((fullpath = realpath(fileinfo.fullpath, canon_path))) {
+                            /* everything has to go through a realpath() */
                             zend_op *dzo = &(dst->opcodes[i]);
                             deallocate(dzo->op1.u.constant.value.str.val);
                             dzo->op1.u.constant.value.str.len = strlen(fullpath);
