@@ -29,12 +29,12 @@
 
  */
 
-/* $Id: apc_globals.h,v 3.59 2007/03/21 21:07:28 rasmus Exp $ */
+/* $Id: apc_globals.h,v 3.66 2007/10/18 20:35:10 rasmus Exp $ */
 
 #ifndef APC_GLOBALS_H
 #define APC_GLOBALS_H
 
-#define APC_VERSION "3.0.14"
+#define APC_VERSION "3.0.15"
 
 #include "apc_cache.h"
 #include "apc_stack.h"
@@ -74,6 +74,9 @@ ZEND_BEGIN_MODULE_GLOBALS(apc)
     apc_optimize_function_t apc_optimize_function;   /* optimizer function callback */
 #ifdef MULTIPART_EVENT_FORMDATA
     zend_bool rfc1867;           /* Flag to enable rfc1867 handler */
+    char* rfc1867_prefix;        /* Key prefix */
+    char* rfc1867_name;          /* Name of hidden field to activate upload progress/key suffix */
+    double rfc1867_freq;         /* Update frequency as percentage or bytes */
 #endif
     HashTable *copied_zvals;     /* my_copy recursion detection list */
 #ifdef ZEND_ENGINE_2
@@ -82,6 +85,8 @@ ZEND_BEGIN_MODULE_GLOBALS(apc)
     zend_bool localcache;        /* enable local cache */
     long localcache_size;        /* size of fast cache */
     apc_local_cache_t* lcache;   /* unlocked local cache */
+    zend_bool force_file_update; /* force files to be updated during apc_compile_file */
+    char canon_path[MAXPATHLEN]; /* canonical path for key data */
 ZEND_END_MODULE_GLOBALS(apc)
 
 /* (the following declaration is defined in php_apc.c) */
