@@ -28,7 +28,7 @@
 
  */
 
-/* $Id: apc_main.c,v 3.97 2007/03/22 16:03:59 gopalv Exp $ */
+/* $Id: apc_main.c,v 3.102 2007/10/05 23:06:56 gopalv Exp $ */
 
 #include "apc_php.h"
 #include "apc_main.h"
@@ -258,7 +258,6 @@ static zend_op_array* cached_compile(zend_file_handle* h,
 
 default_compile:
 
-    cache_entry->autofiltered = 1;
     if(APCG(report_autofilter)) {
         apc_wprint("Autofiltering %s", h->opened_path);
     }
@@ -270,14 +269,16 @@ default_compile:
     }
     
     apc_stack_pop(APCG(cache_stack)); /* pop out cache_entry */
+    
+    apc_cache_release(apc_cache, cache_entry);
 
     /* cannot free up cache data yet, it maybe in use */
     
     zend_llist_del_element(&CG(open_files), h, compare_file_handles); /* XXX: kludge */
     
     h->type = ZEND_HANDLE_FILENAME;
-    
-    return old_compile_file(h, type TSRMLS_CC);
+
+    return NULL;
 }
 /* }}} */
 
@@ -311,7 +312,7 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         return old_compile_file(h, type TSRMLS_CC);
     }
 
-#if PHP_API_VERSION <= 20041225
+#if PHP_API_VERSION < 20041225
 #if HAVE_APACHE && defined(APC_PHP4_STAT)
     t = ((request_rec *)SG(server_context))->request_time;
 #else 
@@ -330,15 +331,20 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         return old_compile_file(h, type TSRMLS_CC);
     }
 
-    if(APCG(localcache)) {
-        /* search for the file in the local cache */
-        cache_entry = apc_local_cache_find(APCG(lcache), key, t);
+
+    if(!APCG(force_file_update)) {
+        if(APCG(localcache)) {
+            /* search for the file in the local cache */
+            cache_entry = apc_local_cache_find(APCG(lcache), key, t);
+        } else {
+            /* search for the file in the cache */
+            cache_entry = apc_cache_find(apc_cache, key, t);
+        }
     } else {
-        /* search for the file in the cache */
-        cache_entry = apc_cache_find(apc_cache, key, t);
+        cache_entry = NULL;
     }
 
-    if (cache_entry != NULL && !cache_entry->autofiltered) {
+    if (cache_entry != NULL) {
         int dummy = 1;
         if (h->opened_path == NULL) {
             h->opened_path = estrdup(cache_entry->data.file.filename);
@@ -346,27 +352,14 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         zend_hash_add(&EG(included_files), h->opened_path, strlen(h->opened_path)+1, (void *)&dummy, sizeof(int), NULL);
         zend_llist_add_element(&CG(open_files), h); /* XXX kludge */
         apc_stack_push(APCG(cache_stack), cache_entry);
-        return cached_compile(h, type TSRMLS_CC);
-    }
-    else if(cache_entry != NULL && cache_entry->autofiltered) {
-        /* nobody else is using this cache_entry */ 
-        if(cache_entry->ref_count == 1) {
-            if(cache_entry->data.file.op_array) {
-                apc_free_op_array(cache_entry->data.file.op_array, apc_sma_free);
-                cache_entry->data.file.op_array = NULL;
-            }
-            if(cache_entry->data.file.functions) {
-                apc_free_functions(cache_entry->data.file.functions, apc_sma_free);
-                cache_entry->data.file.functions = NULL;
-            }
-            if(cache_entry->data.file.classes) {
-                apc_free_classes(cache_entry->data.file.classes, apc_sma_free);
-                cache_entry->data.file.classes = NULL;        
-            }
+        op_array = cached_compile(h, type TSRMLS_CC);
+        if(op_array) {
+            return op_array;
         }
-        /* We never push this into the cache_stack, so we have to do a release */
-        apc_cache_release(apc_cache, cache_entry);
-        return old_compile_file(h, type TSRMLS_CC);
+        if(APCG(report_autofilter)) {
+            apc_wprint("Recompiling %s", h->opened_path);
+        }
+        /* TODO: check what happens with EG(included_files) */
     }
     
     if(apc_cache_busy(apc_cache) && APCG(localcache)) {
@@ -402,6 +395,26 @@ static zend_op_array* my_compile_file(zend_file_handle* h,
         }
     }
 
+    /* Make sure the mtime reflects the files last known mtime in the case of fpstat==0 */
+    if(key.type == APC_CACHE_KEY_FPFILE) {
+        apc_fileinfo_t fileinfo;
+        struct stat *tmp_buf = NULL;
+        if(!strcmp(SG(request_info).path_translated, h->filename)) {
+            tmp_buf = sapi_get_stat(TSRMLS_C);  /* Apache has already done this stat() for us */
+        }
+        if(tmp_buf) { 
+            fileinfo.st_buf = *tmp_buf;
+        } else {
+            if (apc_search_paths(h->filename, PG(include_path), &fileinfo) != 0) {
+#ifdef __DEBUG_APC__
+                fprintf(stderr,"Stat failed %s - bailing (%s) (%d)\n",filename,SG(request_info).path_translated);
+#endif
+                return op_array;
+            }
+        }
+        key.mtime = fileinfo.st_buf.st_mtime;
+    }
+
     HANDLE_BLOCK_INTERRUPTIONS();
 
 #if NONBLOCKING_LOCK_AVAILABLE
@@ -629,14 +642,6 @@ void apc_deactivate(TSRMLS_D)
         apc_cache_entry_t* cache_entry =
             (apc_cache_entry_t*) apc_stack_pop(APCG(cache_stack));
 
-        if (cache_entry->data.file.functions) {
-            for (i = 0; cache_entry->data.file.functions[i].function != NULL; i++) {
-                zend_hash_del(EG(function_table),
-                    cache_entry->data.file.functions[i].name,
-                    cache_entry->data.file.functions[i].name_len+1);
-            }
-        }
-
         if (cache_entry->data.file.classes) {
             for (i = 0; cache_entry->data.file.classes[i].class_entry != NULL; i++) {
 #ifdef ZEND_ENGINE_2
