@@ -26,7 +26,7 @@
 
  */
 
-/* $Id: php_apc.c,v 3.140 2007/03/28 07:14:54 gopalv Exp $ */
+/* $Id: php_apc.c,v 3.151 2007/09/18 21:35:46 shire Exp $ */
 
 #include "apc_zend.h"
 #include "apc_cache.h"
@@ -92,6 +92,7 @@ static void php_apc_init_globals(zend_apc_globals* apc_globals TSRMLS_DC)
     apc_globals->localcache = 0;
     apc_globals->localcache_size = 0;
     apc_globals->lcache = NULL;
+    apc_globals->force_file_update = 0;
 }
 
 static void php_apc_shutdown_globals(zend_apc_globals* apc_globals TSRMLS_DC)
@@ -139,6 +140,28 @@ static PHP_INI_MH(OnUpdateShmSegments) /* {{{ */
 }
 /* }}} */
 
+#ifdef MULTIPART_EVENT_FORMDATA
+static PHP_INI_MH(OnUpdateRfc1867Freq) /* {{{ */
+{
+    int tmp;
+    tmp = zend_atoi(new_value, new_value_length);
+    if(tmp < 0) {
+        apc_eprint("rfc1867_freq must be greater than or equal to zero.");
+        return FAILURE;
+    }
+    if(new_value[new_value_length-1] == '%') {
+        if(tmp > 100) {
+            apc_eprint("rfc1867_freq cannot be over 100%%");
+            return FAILURE;
+        }
+        APCG(rfc1867_freq) = tmp / 100.0;
+    } else {
+        APCG(rfc1867_freq) = tmp;
+    }
+    return SUCCESS;
+}
+/* }}} */
+#endif
 
 #ifdef ZEND_ENGINE_2
 #define OnUpdateInt OnUpdateLong
@@ -169,6 +192,9 @@ STD_PHP_INI_BOOLEAN("apc.write_lock", "1",      PHP_INI_SYSTEM, OnUpdateBool,
 STD_PHP_INI_BOOLEAN("apc.report_autofilter", "0", PHP_INI_SYSTEM, OnUpdateBool,         report_autofilter,zend_apc_globals, apc_globals)
 #ifdef MULTIPART_EVENT_FORMDATA
 STD_PHP_INI_BOOLEAN("apc.rfc1867", "0", PHP_INI_SYSTEM, OnUpdateBool, rfc1867, zend_apc_globals, apc_globals)
+STD_PHP_INI_ENTRY("apc.rfc1867_prefix", "upload_", PHP_INI_SYSTEM, OnUpdateStringUnempty, rfc1867_prefix, zend_apc_globals, apc_globals)
+STD_PHP_INI_ENTRY("apc.rfc1867_name", "APC_UPLOAD_PROGRESS", PHP_INI_SYSTEM, OnUpdateStringUnempty, rfc1867_name, zend_apc_globals, apc_globals)
+STD_PHP_INI_ENTRY("apc.rfc1867_freq", "0", PHP_INI_SYSTEM, OnUpdateRfc1867Freq, rfc1867_freq, zend_apc_globals, apc_globals)
 #endif
 STD_PHP_INI_BOOLEAN("apc.localcache", "0", PHP_INI_SYSTEM, OnUpdateBool, localcache, zend_apc_globals, apc_globals)
 STD_PHP_INI_ENTRY("apc.localcache.size", "512", PHP_INI_SYSTEM, OnUpdateInt, localcache_size,  zend_apc_globals, apc_globals)
@@ -199,7 +225,7 @@ static PHP_MINFO_FUNCTION(apc)
 #else
     php_info_print_table_row(2, "Locking type", "File Locks");
 #endif
-    php_info_print_table_row(2, "Revision", "$Revision: 3.140 $");
+    php_info_print_table_row(2, "Revision", "$Revision: 3.151 $");
     php_info_print_table_row(2, "Build Date", __DATE__ " " __TIME__);
     php_info_print_table_end();
     DISPLAY_INI_ENTRIES();
@@ -503,7 +529,7 @@ int _apc_store(char *strkey, int strkey_len, const zval *val, const unsigned int
     time_t t;
     size_t mem_size = 0;
 
-#if PHP_API_VERSION <= 20041225
+#if PHP_API_VERSION < 20041225
 #if HAVE_APACHE && defined(APC_PHP4_STAT)
     t = ((request_rec *)SG(server_context))->request_time;
 #else
@@ -638,7 +664,7 @@ PHP_FUNCTION(apc_fetch) {
         return;
     }
 
-#if PHP_API_VERSION <= 20041225
+#if PHP_API_VERSION < 20041225
 #if HAVE_APACHE && defined(APC_PHP4_STAT)
     t = ((request_rec *)SG(server_context))->request_time;
 #else 
@@ -795,7 +821,7 @@ PHP_FUNCTION(apc_load_constants) {
 
     if(!strkey_len) RETURN_FALSE;
 
-#if PHP_API_VERSION <= 20041225
+#if PHP_API_VERSION < 20041225
 #if HAVE_APACHE && defined(APC_PHP4_STAT)
     t = ((request_rec *)SG(server_context))->request_time;
 #else 
@@ -863,12 +889,11 @@ PHP_FUNCTION(apc_compile_file) {
     zend_hash_init_ex(&cg_class_table, 10, NULL, ZEND_CLASS_DTOR, 1, 0);
     cg_orig_class_table = CG(class_table);
     CG(class_table) = &cg_class_table;
-    zend_hash_init_ex(&eg_function_table, 100, NULL, ZEND_FUNCTION_DTOR, 1, 0);
     eg_orig_function_table = EG(function_table);
-    EG(function_table) = &eg_function_table;
-    zend_hash_init_ex(&eg_class_table, 10, NULL, ZEND_CLASS_DTOR, 1, 0);
+    EG(function_table) = CG(function_table);
     eg_orig_class_table = EG(class_table);
-    EG(class_table) = &eg_class_table;
+    EG(class_table) = CG(class_table);
+    APCG(force_file_update) = 1;
     
     /* Compile the file, loading it into the cache */
     file_handle.type = ZEND_HANDLE_FILENAME;
@@ -878,19 +903,18 @@ PHP_FUNCTION(apc_compile_file) {
     zend_try {
         op_array = zend_compile_file(&file_handle, ZEND_INCLUDE TSRMLS_CC);
     } zend_catch {
-        apc_eprint("Error compiling %s in apc_compile_file.", filename);
+        apc_wprint("Error compiling %s in apc_compile_file.", filename);
         op_array = NULL;
     } zend_end_try();
 
     /* Return class/function tables to previous states, destroy temp tables */
+    APCG(force_file_update) = 0;
     CG(function_table) = cg_orig_function_table;
     zend_hash_destroy(&cg_function_table);
     CG(class_table) = cg_orig_class_table;
     zend_hash_destroy(&cg_class_table);
     EG(function_table) = eg_orig_function_table;
-    zend_hash_destroy(&eg_function_table);
     EG(class_table) = eg_orig_class_table;
-    zend_hash_destroy(&eg_class_table);
     
     /* Restore global settings */
     APCG(slam_defense) = slam_defense;
@@ -923,6 +947,7 @@ function_entry apc_functions[] = {
 	PHP_FE(apc_define_constants,    NULL)
 	PHP_FE(apc_load_constants,      NULL)
 	PHP_FE(apc_compile_file,        NULL)
+	PHP_FE(apc_add,                 NULL)
 	{NULL, 		NULL,				NULL}
 };
 /* }}} */
